#module  LIB_SECTION_UFO  "V1.1"

/*|++LIBSECUFO.C

 Facility: 
	Fermilab Accelerator Control System - VAX/VMS ACNET
	General utility routine for FERMILIB.

 Abstract:
	This module opens a file for a global or private section.

 Environment:
	Called from user mode (usually) before calling $CRMPSC.
|--*/
/*
 Modification History:
 Author: F. Nagy			Creation date: 11-Oct-83

 V1.0	11-Oct-83  FJN	Created from SNATCHER.C to replace GBLSECUFO.MAR
 V1.1	14-Aug-84  FJN	Eliminated use of LIB_NARGS by small trick
*/

/******************
 * Include Files: *
 ******************/
#include  descrip			/* Descriptor definitions */
#include  descrip2			/* Descriptor extensions */
#include  vaxtypes			/* General VAX-specific types */
#include  $FAB				/* File Access Block */
#include  $XAB				/* Extended Attributes Blocks */

/********************************
 * Local definitions and types: *
 ********************************/
#define  PAGE_SIZE	512		/* Page/disk block size in bytes */

typedef struct dsc$descriptor_s *dscptr; /* Pointer to string descriptor */

					/* Return to caller on failure with
					   the bad status code */
#define RETURN_ON_FAILURE(ss)	if ($VMS_FAILURE(ss)) return ss

/**********************
 * Local Own Storage: *
 **********************/
static struct $FAB sfab;		/* FAB for section file */
static struct $XABFHC sfhc;		/* XAB for file size */

/**********************************************
 * Global Procedure and Function definitions: *
 **********************************************/
ulong SYS$OPEN();			/* Open file */
ulong SYS$CREATE();			/* Create new file and open */

ulong LIB$ANALYZE_SDESC();		/* Analyze string descriptor */


/*+0LIB_SECTION_UFO

 Functional Description:
	This routine opens a file to be used as a private/global section file
	(the channel number returned is used in a later call to $CRMPSC).  An
	RMS $OPEN or $CREATE is performed with user-file-open (UFO) set in the
	file options (FOP) of the File Access Block (FAB).  An existing file
	may be opened or a new file created.

 Input Parameters:
	filnam - pointer to descriptor for file name string.
	dftnam - optional pointer to descriptor for default file name string.
	blkcre - optional file creation block count:
			<0 create new file
			=0 (or non-existant) open existing file readonly
			>0 open existing file, create if no such file
		 absolute value is size of new file in blocks.
	rdonly - optional readonly file flag (if non-zero). By value.

 Implicit Inputs:
	NONE

 Output Parameters:
	R0 - completion status returned.
	chan - address of a longword in which the channel number of the
	       opened file is returned or (if an RMS error occurs) the STV
	       field from the FAB is returned.
	blkopn - optional address of a longword in which the size of the opened
		 file (in blocks) is returned.

 Implicit Outputs:
	NONE

 Condition Codes Returned:
	RMS$_NORMAL	success
	LIB$_INVSTRDES	invalid strin descriptor
	Other errors returned from RMS.

 Side Effects:
	An internal static FAB is used to open the file.  This can be reused
	on subsequent calls but means that this program is non-reentrant.
-*/
/*+ LIB_SECTION_UFO
 This routine opens a file to be used as a private/global section file
 (the channel number returned is used in a later call to $CRMPSC).  An
 RMS $OPEN or $CREATE is performed with user-file-open (UFO) set in the
 file options (FOP) of the File Access Block (FAB).  An existing file
 may be opened (for writing, or readonly) or a new file created.

 status.wlc.v = LIB_SECTION_UFO( filnam.rt.dx, chan.ww.r [,dftnam.rt.dx]
                                 [,blkcre.rl.r] [,rdonly.rl.v]
                                 [,blkopn.wl.r] )

  filnam  file name string.  By descriptor.
  chan    longword in which the channel number of the opened file is
          returned for use by $CRMPSC or, if an RMS error occurs, the
          STV field from the FAB is returned.  By reference.
  dftnam  optional default file name string.  By descriptor.
  blkcre  optional file creation block count:
            <0 create new file 
            =0 (or non-existant) open existing file
            >0 open existing file, create if no such file
          The absolute value is size of new file in blocks.  By reference.
  rdonly  if non-zero, opens existing file for readonly access (only if
          the blkcre parameter is 0 or non-existant).  By value.
  blkopn  optional longword in which the size of the opened file (in
          blocks) is returned.  By reference.

 An internal static FAB is used to open the file.  This can be reused
 on subsequent calls but means that this program is non-reentrant.
-*/



#define NARG_DFTNAM	3		/* Default file spec. argument no. */
#define NARG_BLKCRE	4		/* File creation flag/size arg. no. */
#define NARG_RDONLY	5		/* Readonly file open flag */
#define NARG_BLKOPN	6		/* File size return argument no. */

LIB_SECTION_UFO( d_filnam, p_chan, d_dftnam, p_blkcre, rdonly, p_blkopn )
dscptr d_filnam;			/* File name string descriptor */
int *p_chan;				/* Channel number return */
dscptr d_dftnam;			/* Default file name string */
int *p_blkcre;				/* File create flag/size */
int rdonly;				/* Readonly file flag (if no create) */
int *p_blkopn;				/* Returns file size */
{
register ulong status;			/* Completion status return */
register int na;			/* Number of call arguments */
register int blkcre;			/* File creation flag/size */
register int blkopn = 0;		/* Return file size flag */
register int justread = 0;		/* Processed readonly flag */
uword length;				/* Holder for string lengths */

na = *( (long *) &d_filnam - 1);	/* Get number of call arguments */

/*
  Setup optional argument flags.
*/
blkopn = ((na >= NARG_BLKOPN) && (p_blkopn != NULL));
blkcre = ((na >= NARG_BLKCRE) && (p_blkcre != NULL)) ? *p_blkcre : 0;
justread = ((blkcre != 0) && (na >= NARG_RDONLY) && (rdonly != 0));

/*
  Initialize the File Access Block for UFO.
*/
sfab.fab$b_bid = FAB$C_BID;		/* Block ID and length */
sfab.fab$b_bln = FAB$C_BLN;

sfab.fab$l_dna = 0;			/* Prepare for NO default file spec */
sfab.fab$b_dns = 0;

sfab.fab$w_ifi = 0;			/* Clear the IFI also */

sfab.fab$l_fop = FAB$M_UFO;		/* User open (for sections) */

if (blkcre == 0)
    sfab.fab$b_fac = 			/* Open for reading or writing */
    	justread ? FAB$M_GET : FAB$M_PUT;
else
    {
    sfab.fab$b_fac = FAB$M_PUT;		/* Open for writing */
    sfab.fab$l_fop |= FAB$M_CBT;	/* User open (for sections) and
					   contiguous-best-try allocation */
    sfab.fab$l_alq = ABS(blkcre);	/* Allocate user-specified space */
    if (blkcre > 0)
	sfab.fab$l_fop |= FAB$M_CIF;	/* Open existing file/create if none */
    sfab.fab$b_org = FAB$C_SEQ;		/* Sequential file */
    sfab.fab$b_rfm = FAB$C_FIX;		/* Of fixed length records */
    sfab.fab$w_mrs = PAGE_SIZE;		/* Each 1 block long */
    }

sfab.fab$b_shr = justread ? FAB$M_GET : FAB$M_NIL;

/*
  If opened file size is to be returned, setup XABFHC block.
*/
if (blkopn)
    {
    sfab.fab$l_xab = &sfhc;		/* Point to FHC XAB */
    sfhc.xab$b_cod = XAB$C_FHC;		/* Block ID and length */
    sfhc.xab$b_bln = XAB$C_FHCLEN;
    sfhc.xab$l_nxt = NULL;		/* Single XAB in chain */
    }

/*
  Store pointers and lengths of the file specification and default file
  specification strings (if any).
*/
status = LIB$ANALYZE_SDESC( d_filnam, &length, &sfab.fab$l_fna);
RETURN_ON_FAILURE( status);
sfab.fab$b_fns = length;

if ((na >= NARG_DFTNAM) && (d_dftnam != NULL))
    {
    status = LIB$ANALYZE_SDESC( d_dftnam, &length, &sfab.fab$l_dna);
    RETURN_ON_FAILURE( status);
    sfab.fab$b_dns = length;
    }

/*
  Open/create the file and return the status from RMS.
*/
status = blkcre ? SYS$CREATE( &sfab) : SYS$OPEN( &sfab);
*p_chan = sfab.fab$l_stv;
if (blkopn)				/* Get EOF block number as file size */
    *p_blkopn = (sfhc.xab$w_ffb == 0) ? sfhc.xab$l_ebk - 1 : sfhc.xab$l_ebk;

return status;
}
