#module  FORT_DETAB "V2.0"

/*
 Facility: 
    General VAX utility program

 Abstract:
    Remove tab characters from VAX or PDP-11 FORTRAN source files,
    replacing them with spaces.

 Environment:
    Foreign DCL command.

 Author: Frank J. Nagy		Fermilab Research Division EED/Controls
 Modification History:

  V1.0	01-Aug-84  FJN	Created
  V2.0	07-Sep-86  FJN	Modified for VAX native mode using VAX C V2.x
*/

#ifdef vaxc
#include stdio
#else
#include <stdio.h>
#endif

#define	FALSE	0
#define	TRUE	1

#define	EOS	'\0'
#define	BLANK	' '
#define	TAB	'\t'
#define	NEWLINE	'\n'

static char	line[133];		/* Character line buffer */

extern char	*strchr();		/* String search function */

static int	rflag = FALSE;		/* Set TRUE if "-r" option given to use
					   right-justified labels */
static int	xflag = FALSE;		/* Set TRUE if "-x" option given to
					   eliminate columns >72 of input */

#define	CONTCOL		5		/* Continuation column in "line" */
#define	FIRSTCOL	6		/* 1st code column in "line" */
#define TOSSCOL		72		/* First column tossed by "-x" */


main(argc, argv)
char *argv[];
int argc;
{
register int	i, j;
register char	*lstart, *lend;		/* General string pointers */
char	lterm;				/* Label terminator character */
long	alabel;				/* Statement label holder */
long	linecount;			/* Input line counter */

lterm = 0;
/*
  Process the command line arguments by re-opening the standard
  input and output files.  Also handle the option flags.
*/
for (i = 1; i < argc; i++)
    if (argv[i][0] == '-')
	for (j = 1; argv[i][j] != EOS; j++)
	    if (tolower(argv[i][j]) == 'r')
		rflag++;		/* Set flag for "-r" */
	    else if (tolower(argv[i][j]) == 'x')
		xflag++;		/* Set flag for "-x" */
	    else
		fprintf( stderr, "Unknown option '%c'\n", argv[i][j]);

    else if (lterm == 0)
	{
	if (freopen(argv[i], "r", stdin) == NULL)
	    {
	    perror(argv[i]);
	    exit(1);
	    }
	lterm++;
	}
    else if (lterm == 1)
	{
#ifdef vaxc
	if (freopen(argv[i], "w", stdout, "rfm=var", "rat=cr") == NULL)
#else
	if (freopen(argv[i], "w", stdout) == NULL)
#endif
	    {
	    perror(argv[i]);
	    exit(1);
	    }
	lterm++;
	}

linecount = 0;				/* Initialize line counter */
/*
  Read input file line-by-line until EOF.
*/
while (gets(line) != NULL)
    {
    linecount++;			/* Count of lines */

    /*
      If "-x" used, then make sure input line string stops after
      column 72.
    */
    if (xflag)
	line[TOSSCOL] = EOS;

    /*
      Get pointer to end of string and work backwards over
      trailing spaces and tabs.
    */
    for (lend = line; *lend != EOS; lend++);
    if (lend != line)
	{
	lend--;
	while ((*lend == BLANK) | (*lend == TAB))
	    lend--;
	*(++lend) = EOS;
	}

    /*
      Now handle the rules for tab replacement specific to
      PDP-11 and VAX FORTRAN sources.
    */
    switch (line[0])
	{
	case EOS:			/* Blank line */
	printf( "\n");
	break;

	case TAB:			/* First character is tab */
	switch (line[1])
	    {
	    case EOS:			/* Just a tab => blank line */
	    printf( "\n");
	    break;

	    case '1':			/* <tab><digit> is continuation */
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    case '8':
	    case '9':
	    printf( "     %c", line[1]);
	    if (line[2] == TAB)
		{
		printf( "          ");
		NormalRules( &line[3], 17);
		}
	    else
		NormalRules( &line[2], 7);
	    break;

	    case TAB:			/* <tab><tab>... statement line */
	    NormalRules( line, 1);
	    break;

	    default:			/* Others as normal statements */
	    printf( "      ");
	    NormalRules( &line[1], 7);
	    break;
	    }
	break;

	case '0':			/* Statement label begins line */
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	sscanf( line, "%ld%c", &alabel, &lterm);
	if (rflag)
	    printf( "%5ld ", alabel);
	else
	    printf( "%-5ld ", alabel);
	lstart = strchr( line, lterm);
	if (lterm == TAB)
	    if (lstart++ == TAB)
		NormalRules( lstart, 9);
	    else
		NormalRules( lstart, 7);
	else
	    NormalRules( &line[FIRSTCOL], 7);
	break;

	default:			/* Comments, etc. */
	NormalRules( line, 1);
	break;
	}
    }

fprintf( stderr, "  %ld lines processed\n", linecount);
}		

/*
  The following function implements normal tab replacement rules for
  remainder of the line (tab stops every 8 columns).
*/
NormalRules( lstart, loffset )
char	*lstart;			/* Pointer to string */
int	loffset;			/* Column number offset of string */
{
register char	*lend;			/* String pointer */
register int	nn;

while ((lend = strchr(lstart, TAB)) != NULL)
    {
    /*
      Found a tab, output string up to tab.
    */
    *lend++ = EOS;
    printf( "%s", lstart);

    /*
      Output spaces to replace the tab itself.
    */
    nn = 8 - ((lend - lstart - loffset) & 07);
    while (--nn >= 0)
	putchar(BLANK);
    lstart = lend;
    }

/*
  Output the remainder of the line, followed by a line terminator.
*/
printf( "%s\n", lstart);
}
